function [FilteredI, fiz, SmoothingCoeffsMatrixN] = Optimizing_Parameters_MamC3(ImageR,ImageO,Delta,D,FLSParams)   


FLSParams
[a,b,c] = size(ImageR);
FilteredI = ImageR;
NDatosWindow = (2*Delta+1)^2;
PosCenter = ceil(NDatosWindow/2);


SmoothingCoeffsMatrixN = [];
%sigma_N = [];



%General setting for FIS
fiz = mamfis("AndMethod","prod","OrMethod","probor",...
"ImplicationMethod", "prod","AggregationMethod", "probor");


%INPUTs

fiz = addInput(fiz,[0 200],'Name',"D1");
fiz = addMF(fiz,"D1","gauss2mf",[-5000 0 FLSParams(1) FLSParams(2)],'Name',"Low");
fiz = addMF(fiz,"D1","gauss2mf",[FLSParams(3) FLSParams(4) 5000 1],'Name',"High");


fiz = addInput(fiz,[0 200],'Name',"D2");
fiz = addMF(fiz,"D2","gauss2mf",[-5000 0 FLSParams(5) FLSParams(6)],'Name',"Low");
fiz = addMF(fiz,"D2","gauss2mf",[FLSParams(7) FLSParams(8) 5000 1],'Name',"High");

%OUTPUTs

fiz = addOutput(fiz,[0 1],'Name',"C3");
fiz = addMF(fiz,"C3","gauss2mf",[-5000 0 FLSParams(9) FLSParams(10)],'Name',"Low");
fiz = addMF(fiz,"C3","gauss2mf",[FLSParams(11) FLSParams(12) 5000 1],'Name',"High");


% Rulelist, The first and second columns are for the first, second inputs.
% The third column is for outputs. 
% The last two coloumns are for the weight of the rule.
% The last coulumn is for the connections ( 1 == and, 2  == or ).
%( zero == ignore this subset ). Negative is for the no conection.

ruleList = ([1  1  1  1  1;
             2  2  2  1  2 ]); 



fiz = addRule(fiz, ruleList);


%  Extract the information of the fuzzy system 

%      fiz
%      fiz.Rules
%      fiz.Inputs(1)
%      fiz.Inputs(2)
%      fiz.Outputs(1)
%      fiz.Inputs(1).MembershipFunctions
%      fiz.Inputs(2).MembershipFunctions
%      fiz.Outputs(1).MembershipFunctions

% Plot the MFs for the inputs and the output


%      set(groot, 'DefaultLineLineWidth', 2);
%      set(groot, 'DefaultTextFontWeight', 'bold'); 
%      set(groot, 'DefaultTextFontSize', 12); 
%      fig_handle = figure;
%  	 set(fig_handle, 'Position', [100 100 1600 500])
%  
%      subplot(1, 3 , 1)
%      plotmf(fiz, 'input',1)
% 
%      subplot(1, 3 , 2)
%      plotmf(fiz, 'input',2)
%   
%      
%      subplot(1, 3 , 3)
%      plotmf(fiz, 'output', 1)
% 
%      print('-depsc', '-r300', 'InOutN30_DC3_MFs.eps');




for i=(Delta+1):1:(a-Delta)
    for j=(Delta+1):1:(b-Delta)
        
        %Noisy data
        WindowRR = ImageR(i-Delta:i+Delta,j-Delta:j+Delta,1);
        WindowRG = ImageR(i-Delta:i+Delta,j-Delta:j+Delta,2);
        WindowRB = ImageR(i-Delta:i+Delta,j-Delta:j+Delta,3);
        
        %build data matrix to perform later eigenvector analysis
        VectorRR = double(Matrix2Vector(WindowRR));
        VectorRG = double(Matrix2Vector(WindowRG));
        VectorRB = double(Matrix2Vector(WindowRB));
        %means
        MeanR = mean(VectorRR);
        MeanG = mean(VectorRG);
        MeanB = mean(VectorRB);
        %Matrix
        DatosR = double([VectorRR; VectorRG; VectorRB ]');
        
        %Color vector processed in this iteration

        ColorVectorIJ = [DatosR(PosCenter,1) DatosR(PosCenter,2) DatosR(PosCenter,3)];        
        
        %Center data around means
        DatosR = double([(VectorRR - MeanR); (VectorRG - MeanG); (VectorRB - MeanB) ]');
                
        %Move to EIG space (reference system)
        %EIG: eigenvectors and eigenvalues
        %Solves Eqs (1-3) in the paper
        [EIGVEC EIGVAL] = eig(cov(DatosR));

        
        %Rotate data to EIGVEC reference system
        %Eq (4) in the paper
        DatosNuevosR = double(DatosR)*EIGVEC;

        %Vector to filter in EIGVEC space
        VectorSVD = [ DatosNuevosR(PosCenter,1) DatosNuevosR(PosCenter,2) DatosNuevosR(PosCenter,3)];

        %Compute standard deviation of each variable in EIGVEC space 
        %In the paper these are sigma(U^{i,j,k}) 
        stdDatosNuevosR = std(DatosNuevosR);


        %------%

        %Save the SD for noisy image

        %sigma_N = [sigma_N; stdDatosNuevosR]; 
        %save('sigma_N.mat','sigma_N')

        %------%

        %Normalize standard deviations: Eq (7) in the paper.       
        sumstdDatosNuevosR = sum(stdDatosNuevosR);
        if ( sumstdDatosNuevosR == 0) %it is unlikely but just in case...
            stdNormDatosNuevosR = [1/3 1/3 1/3];
        else
            stdNormDatosNuevosR = stdDatosNuevosR ./ sumstdDatosNuevosR; %Eq (7)
        end
        %add tiny amount just in case one of them is zero
        stdNormDatosNuevosR = stdNormDatosNuevosR + 0.0000000001;

        d1 = abs(stdDatosNuevosR(1) - stdDatosNuevosR(2));
        d2 = abs(stdDatosNuevosR(2) - stdDatosNuevosR(3));
 
        SmoothingCoeffs = [0, 0, evalfis(fiz,[(d1),(d2)])];

        %------%
        
        %Save the smoothing coffecients

        SmoothingCoeffsMatrixN = [SmoothingCoeffsMatrixN; SmoothingCoeffs];
        %save('SmoothingCoeffsMatrixN.mat', 'SmoothingCoeffsMatrixN');

        %-------%

        
        
        %Processing in EIGVEC space Eq. (6)
        %First compute weights in Eq (8)
        %Weights, Numerator and denominator for each channel
        
        NumD1 = 0;
        DenD1 = 0;
        NumD2 = 0;
        DenD2 = 0;
        NumD3 = 0;
        DenD3 = 0;
        WeightsD1 = [];
        WeightsD2 = [];
        WeightsD3 = [];
        %loop through the data
        for k=1:1:NDatosWindow
            Weight = 1/exp(abs(VectorSVD(1)-DatosNuevosR(k,1))/(D/(stdNormDatosNuevosR(1)* SmoothingCoeffs(1))));
            WeightsD1 = [WeightsD1 Weight];
            NumD1 =  NumD1 + Weight.*DatosNuevosR(k,1);
            DenD1 = DenD1 + Weight; 

            Weight = 1/exp(abs(VectorSVD(2)-DatosNuevosR(k,2))/(D/(stdNormDatosNuevosR(2)* SmoothingCoeffs(2))));
            WeightsD2 = [WeightsD2 Weight];
            NumD2 =  NumD2 + Weight.*DatosNuevosR(k,2);
            DenD2 = DenD2 + Weight; 

            Weight = 1/exp(abs(VectorSVD(3)-DatosNuevosR(k,3))/(D/(stdNormDatosNuevosR(3)* SmoothingCoeffs(3))));
            WeightsD3 = [WeightsD3 Weight];
            NumD3 =  NumD3 + Weight.*DatosNuevosR(k,3);
            DenD3 = DenD3 + Weight; 
            
        end
        %Cocient
        VectorSVDFiltrado = [NumD1/DenD1 NumD2/DenD2 NumD3/DenD3];
        
        %Back to RGB from EIGVEC space Eq. (5): inverse rotation plus add
        %means 
        ColorVectorIJFiltrado = round(VectorSVDFiltrado*inv(EIGVEC) + [MeanR MeanG MeanB]);
        
        FilteredI(i,j,:) = ColorVectorIJFiltrado;

        %------%

        %save('FilteredImage.mat' , 'FilteredI')

        %------%   
    end
    
end


PSNR = PSNRImages(FilteredI, ImageO)